#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.


class SmartCursor():
    # Smart cursor of LORZE erasandcad

    def __init__(self, geohelp):
        # geohelp object
        self.__geohelp= geohelp

        # list with visible elements, [(element, key), ..]
        self.__elements= []

        # type of point mark, 'P'= point, 'S'= segment
        self.__type= ''

        # x & y coordinate of point
        self.__x, self.__y= 0.0, 0.0

        # id
        self.__lrzid=''

        # key of the drawing
        self.__drawkey=''

        # smart cursor point
        self.__spx, self.__spy, self.__tol= 0.0, 0.0, 0.0

        # smart cursor rect
        self.__scx, self.__scy, self.__scw, self.__sch= 0.0, 0.0, 0.0, 0.0


    def AddElement(self, element, key):
        # add lorze darawing element to a list
        self.__elements.append((element, key))


    def GetElements(self):
        # gets all elements in list
        return(self.__elements)


    def Get(self, px, py, tolerance):
        # search element in the smart cursor, x, y= point coordinates, toreance= value
        found= None
        self.__type, self.__lrzid, self.__drawkey= '', '', ''
        self.__x, self.__y= 0.0, 0.0
        self.__spx, self.__spy, self.__tol= px, py, tolerance
        self.__scx= self.__spx- (self.__tol/ 2.0)
        self.__scy= self.__spy- (self.__tol/ 2.0)
        self.__scw, self.__sch= self.__tol, self.__tol

        for i in self.__elements:
            # element, drawkey, rect coordinates
            element, key= i
            x, y, w, h= element.GetRec()

            if self.__geohelp.RectInRect((self.__scx, self.__scy, self.__scw, self.__sch), (x, y, w, h)):
                # smart cursor in rect, found: (type, x, y, lrzid) or None
                if element.GetId()[0]== 'L':
                    # line
                    found= self.GetLine(element)

                if found!= None:
                    break

        if found!= None:
            # point found
            self.__type, self.__x, self.__y, self.__lrzid= found
            self.__drawkey= key
            return(True)
        else:
            # no point found
            return(False)


    def GetType(self):
        return(self.__type)


    def GetCoord(self):
        return(self.__x, self.__y)


    def GetLrzId(self):
        return(self.__lrzid)


    def GetKey(self):
        return(self.__drawkey)


    def GetX(self):
        return(self.__x)


    def GetY(self):
        return(self.__y)


    def GetLine(self, element):
        # search smart cursor, set no point found
        found= None

        # line: get rect
        x1, y1, x2, y2= element.GetCoords()

        if self.__geohelp.PointInRect((x1, y1), (self.__scx, self.__scy, self.__scw, self.__sch)):
            # point 1 found
            found= ('P', x1, y1, element.GetId())
        elif self.__geohelp.PointInRect((x2, y2), (self.__scx, self.__scy, self.__scw, self.__sch)):
            # point 2 found
            found= ('P', x2, y2, element.GetId())
        else:
            # search segment
            coords= self.__geohelp.PointOverSegment((x1, y1, x2, y2), (self.__spx, self.__spy), self.__tol)
            if coords!= None:
                # segment found
                found= ('S', coords[0], coords[1], element.GetId())

        return(found)


if __name__== '__main__':
    from lrzdrawline import DrawLine
    from lrzgeohelp import LorzeGeoHelp
    from lrzoptions import LorzeOptions
    from lrzvalchck import LorzeValueCheck

    valchck= LorzeValueCheck()
    options= LorzeOptions()
    geohelp= LorzeGeoHelp(options)

    print('Set line (L0, 20, 20, 10, 10, black, test, SOLID, 1 mm)')
    line= DrawLine(geohelp)
    line.SetId('L0')
    line.SetCoords(20, 20, 10, 10)
    line.SetColor('black')
    line.SetLayer('test')
    line.SetStyle('Solid')
    line.SetWidth('1 mm')

    print('Set smart cursor object')
    smartcurs= SmartCursor(geohelp)
    print('Add line object with key "main"')
    smartcurs.AddElement(line, 'main')
    print('Get Elements', smartcurs.GetElements())

    print('reset Smart Cursor, set to 2, 2 with tolerance 4')
    print(smartcurs.Get(2, 2, 4))

    print('Set Smart Cursor to 15, 15 with tolerance 4')
    print(smartcurs.Get(15, 15, 4))
    print(smartcurs.GetType(), smartcurs.GetCoord(), smartcurs.GetLrzId(), smartcurs.GetKey())

    print('Set Smart Cursor from 20, 20 with tolerance 4')
    print(smartcurs.Get(20, 20, 4))
    print(smartcurs.GetType(), smartcurs.GetCoord(), smartcurs.GetLrzId(), smartcurs.GetKey())

    print('Set Smart Cursor from 10, 10 with tolerance 4')
    print(smartcurs.Get(10, 10, 4))
    print(smartcurs.GetType(), smartcurs.GetCoord(), smartcurs.GetLrzId(), smartcurs.GetKey())

